@@ -81,18 +81,21 @@ impl Crc {
         let mut words = data.chunks_exact(4);
         for word in words.by_ref() {
             let word = u32::from_be_bytes(word.try_into().unwrap());
-            self.reg.dr_mut().write(|w| w.dr().bits(word));
+            // todo: Put back once PAC settles. Currently causing error on H7
+            // self.reg.dr_mut().write(|w| w.dr().bits(word));
         }
 
         // there will be at most 3 bytes remaining, so 1 half-word and 1 byte
         let mut half_word = words.remainder().chunks_exact(2);
         if let Some(half_word) = half_word.next() {
             let half_word = u16::from_be_bytes(half_word.try_into().unwrap());
-            self.reg.dr16_mut().write(|w| w.dr16().bits(half_word));
+            // todo: Put back once PAC settles. Currently causing error on H7
+            // self.reg.dr16_mut().write(|w| w.dr16().bits(half_word));
         }
 
         if let Some(byte) = half_word.remainder().first() {
-            self.reg.dr8_mut().write(|w| w.dr8().bits(*byte));
+            // todo: Put back once PAC settles. Currently causing error on H7
+            // self.reg.dr8_mut().write(|w| w.dr8().bits(*byte));
         }
     }
 
